@@ -1,3 +1,4 @@
+from tkinter.ttk import Separator
 import typer, requests, os, zipfile, glob, shutil, json, sys, time, logging
 from pathlib import Path
 import snow_pyrepl as pyrepl
@@ -8,7 +9,7 @@ import datauri
 from watchdog.observers import Observer
 from watchdog.events import LoggingEventHandler
 
-__version__ = "1.2.3"
+__version__ = "1.2.4"
 homedir = Path.home()
 homedir = str(homedir).replace("\\", "/")
 
@@ -34,7 +35,7 @@ def get_json(user, replname, sid):
 	return r.json()['id']
 
 
-def downloadRepl(user, replname):
+def downloadRepl(user, replname, initialPath=".", noWriteDir=False):
 	def clean(output, join=False):
 		outputList = output
 		try:
@@ -59,10 +60,10 @@ def downloadRepl(user, replname):
 	
 	files = channel.get_output({
 		"exec": {
-			"args": ["ls"]
+			"args": ['sh', '-c', f"cd {initialPath}; ls"]
 		}
 	})
-	
+
 	files = clean(files).split("\n")
 	if "__pycache__" in files:
 		files.remove("__pycache__")
@@ -125,8 +126,17 @@ def downloadRepl(user, replname):
 	
 			original = content
 			content = clean(content)
-	
-			filename = replname+"/"+newfile
+
+			if initialPath == ".":
+				seperator = ""
+			else:
+				seperator = f"{initialPath}/"
+
+			if not noWriteDir:
+				filename = replname+"/"+seperator + newfile
+			else:
+				filename = seperator + newfile
+
 			os.makedirs(os.path.dirname(filename), exist_ok=True)
 	
 			if ".png" in newfile or ".jpg" in newfile or ".webp" in newfile or ".gif" in newfile or ".jpeg" in newfile:
@@ -149,6 +159,77 @@ def downloadRepl(user, replname):
 	print("Client closed")
 
 
+def downloadIndividualFile(user, replname, file, noWriteDir=False):
+	def clean(output, join=False):
+		outputList = output
+		try:
+			if join:
+				output = str(output).split('output: "')[1].split("\"\n, channel:")[0] + str(output).split('output: "')[-1].split("\"\n, channel:")[0]
+			else:
+				output = str(output).split('output: "')[1].split("\"\n, channel:")[0]
+		except:
+			output = ""
+		output = bytes(output, "utf-8").decode("unicode_escape").strip()
+		return output
+	
+	key = __sid__
+	
+	token, url = pyrepl.get_token(user, replname, key)
+	id = get_json(user, replname, key)
+	
+	client = pyrepl.Client(token, id, url)
+	print("Client instantiated")
+	channel = client.open("exec", 'execer')
+	print("Channel opened")
+	
+	newfile = file
+	print(f"REFRESHING FILE/DIR FROM SERVER - {newfile}")
+
+	if ".png" in newfile or ".jpg" in newfile or ".webp" in newfile or ".gif" in newfile or ".jpeg" in newfile:
+		ext = newfile.split(".")[-1]
+		cmd = """bash -c 'function img-data() {   TYPE=$(file --mime-type -b "$1");   ENC=$(base64 "$1");   echo "data:$TYPE;base64,$ENC"; }; img-data \""""+newfile+"""\"'"""
+		content = channel.get_output({
+			"exec": {
+				"args": ["bash", "-c", cmd]
+			}
+		})
+
+	else:
+		content = channel.get_output({
+			"exec": {
+				"args": ["sh", "-c", f"cat '{newfile}'"]
+			}
+		})
+
+	original = content
+	content = clean(content)
+
+	if noWriteDir:
+		filename = newfile
+	else:
+		filename = replname+"/"+newfile
+
+	if ".png" in newfile or ".jpg" in newfile or ".webp" in newfile or ".gif" in newfile or ".jpeg" in newfile:
+		try:
+			uri = datauri.parse(content)
+		except Exception as e:
+			content = clean(original, join=True)
+			uri = datauri.parse(content)
+
+			
+		fd = open(filename, 'wb')
+		fd.write(uri.data)
+		fd.close()
+
+	else:
+		with open(filename, "w+") as f:
+			f.write(content)
+			
+	client.close()
+	print("Client closed")
+
+
+
 @app.callback()
 def callback(ctx: typer.Context):
 	cmd = ctx.invoked_subcommand
@@ -163,7 +244,11 @@ def version():
 	typer.echo(__version__)
 
 @app.command(help="Clone a Repl's contents to your local machine")
-def clone(repl:str=typer.Argument("", help="The URL of the repl which is to be cloned.", show_default=False)):
+def clone(
+		repl:str=typer.Argument("", help="The URL of the repl which is to be cloned.", show_default=False),
+		dir:str=typer.Option("", help="Optionally only clone a single directory", show_default=False),
+		file:str=typer.Option("", help="Optionally specify a specific file to download", show_default=False)
+):
 	if not repl:
 		typer.echo("No Repl was provided.")
 		return
@@ -171,7 +256,14 @@ def clone(repl:str=typer.Argument("", help="The URL of the repl which is to be c
 	user = repl.split("/")[0]
 	replname = repl.split("/")[1]
 	slug = repl
-	downloadRepl(user, replname)
+
+	if dir: 
+		downloadRepl(user, replname, dir)
+	elif file:
+		downloadIndividualFile(user, replname, file)
+	else:
+		downloadRepl(user, replname)
+
 	f = open(f"{replname}/.replitcliconfig", "w")
 	r = requests.get(f"https://replit.com/data/repls/@{slug}", cookies={"connect.sid": __sid__}).json()
 	id = r['id']
@@ -180,7 +272,7 @@ def clone(repl:str=typer.Argument("", help="The URL of the repl which is to be c
 	typer.echo(f"Cloned Repl {replname} to /{replname}")
 
 @app.command(help="Pull the remote contents of the repl inside the working directory.")
-def pull(override:bool=False):
+def pull(override:bool=False, dir:str=typer.Option("", help="Optionally only pull a single directory", show_default=False), file:str=typer.Option("", help="Optionally specify a specific file to download", show_default=False)):
 	if override:
 		f = open(".replitcliconfig", "r")
 		content = f.read()
@@ -196,7 +288,12 @@ def pull(override:bool=False):
 			except:
 				os.remove(file)
 
-		downloadRepl(user, replname)
+		if dir:
+			downloadRepl(user, replname, initialPath=dir, noWriteDir=True)
+		elif file:
+			downloadIndividualFile(user, replname, file=file, noWriteDir=True)
+		else:
+			downloadRepl(user, replname, noWriteDir=True)
 
 		f = open(f".replitcliconfig", "w")
 		slug = url.split("/@")[-1]
@@ -212,7 +309,14 @@ def pull(override:bool=False):
 		slug = url.split("/@")[-1]
 		user = slug.split("/")[0]
 		replname = slug.split("/")[1]
-		downloadRepl(user, replname)
+		
+		if dir:
+			downloadRepl(user, replname, initialPath=dir, noWriteDir=True)
+		elif file:
+			downloadIndividualFile(user, replname, file=file, noWriteDir=True)
+		else:
+			downloadRepl(user, replname, noWriteDir=True)
+
 	typer.echo(f"Refreshed Local Repl Dir")
 
 
